---
title: jq
type: processor
status: stable
categories: ["Mapping"]
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     lib/processor/jq.go
-->

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';


Transforms and filters messages using jq queries.


<Tabs defaultValue="common" values={[
  { label: 'Common', value: 'common', },
  { label: 'Advanced', value: 'advanced', },
]}>

<TabItem value="common">

```yaml
# Common config fields, showing default values
label: ""
jq:
  query: .
```

</TabItem>
<TabItem value="advanced">

```yaml
# All config fields, showing default values
label: ""
jq:
  query: .
  raw: false
  output_raw: false
```

</TabItem>
</Tabs>

:::note Try out Bloblang
For better performance and improved capabilities try out native Benthos mapping with the [bloblang processor](/docs/components/processors/bloblang).
:::

The provided query is executed on each message, targeting either the contents
as a structured JSON value or as a raw string using the field `raw`,
and the message is replaced with the query result.

Message metadata is also accessible within the query from the variable
`$metadata`.

This processor uses the [gojq library][gojq], and therefore does not require
jq to be installed as a dependency. However, this also means there are some
differences in how these queries are executed versus the jq cli which you can
[read about here][gojq-difference].

If the query does not emit any value then the message is filtered, if the query
returns multiple values then the resulting message will be an array containing
all values.

The full query syntax is described in [jq's documentation][jq-docs].

## Error Handling

Queries can fail, in which case the message remains unchanged, errors are
logged, and the message is flagged as having failed, allowing you to use
[standard processor error handling patterns](/docs/configuration/error_handling).

## Fields

### `query`

The jq query to filter and transform messages with.


Type: `string`  
Default: `"."`  

### `raw`

Whether to process the input as a raw string instead of as JSON.


Type: `bool`  
Default: `false`  

### `output_raw`

Whether to output raw text (unquoted) instead of JSON strings when the emitted values are string types.


Type: `bool`  
Default: `false`  

## Examples

<Tabs defaultValue="Mapping" values={[
{ label: 'Mapping', value: 'Mapping', },
]}>

<TabItem value="Mapping">


When receiving JSON documents of the form:

```json
{
  "locations": [
    {"name": "Seattle", "state": "WA"},
    {"name": "New York", "state": "NY"},
    {"name": "Bellevue", "state": "WA"},
    {"name": "Olympia", "state": "WA"}
  ]
}
```

We could collapse the location names from the state of Washington into a field `Cities`:

```json
{"Cities": "Bellevue, Olympia, Seattle"}
```

With the following config:

```yaml
pipeline:
  processors:
    - jq:
        query: '{Cities: .locations | map(select(.state == "WA").name) | sort | join(", ") }'
```

</TabItem>
</Tabs>

[gojq]: https://github.com/itchyny/gojq
[gojq-difference]: https://github.com/itchyny/gojq#difference-to-jq
[jq-docs]: https://stedolan.github.io/jq/manual/

